# Generated by Django 2.2.4 on 2019-08-17 18:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0007_auto_20190817_1814'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='assignedprotection',
            name='active',
        ),
        migrations.RemoveField(
            model_name='assignedprotection',
            name='instant_fail',
        ),
    ]
